@@ -2,7 +2,7 @@
 
 echo 'enum {'
 echo 'SEN_printargs = 0,'
-    sed -n '/printargs/! s/.*SEN(\([^)]*\)).*/\1/p' |
-    sort -u |
+    sed -r -n '/printargs/! s/.*SEN\(([^)]+)\).*/\1/p' |
+    LC_COLLATE=C sort -u |
     sed 's/.*/SEN_&,/'
 echo '};'
