@@ -4,6 +4,7 @@ extern const struct xlat btrfs_balance_ctl_cmds[];
 extern const struct xlat btrfs_balance_flags[];
 extern const struct xlat btrfs_balance_state[];
 extern const struct xlat btrfs_compress_types[];
+extern const struct xlat btrfs_cont_reading_from_srcdev_mode[];
 extern const struct xlat btrfs_defrag_flags[];
 extern const struct xlat btrfs_dev_replace_cmds[];
 extern const struct xlat btrfs_dev_replace_results[];
@@ -35,6 +36,8 @@ extern const struct xlat mtd_type_options[];
 extern const struct xlat mtd_flags_options[];
 extern const struct xlat mtd_otp_options[];
 extern const struct xlat mtd_nandecc_options[];
+extern const struct xlat perf_ioctl_cmds[];
+extern const struct xlat perf_ioctl_flags[];
 extern const struct xlat mq_attr_flags[];
 extern const struct xlat sigev_value[];
 extern const struct xlat adjtimex_modes[];
@@ -56,6 +59,7 @@ extern const struct xlat sg_io_dxfer_direction[];
 extern const struct xlat sg_io_flags[];
 extern const struct xlat sigaltstack_flags[];
 extern const struct xlat iffflags[];
+extern const struct xlat arp_hardware_types[];
 extern const struct xlat v4l2_pix_fmts[];
 extern const struct xlat v4l2_sdr_fmts[];
 extern const struct xlat v4l2_device_capabilities_flags[];
@@ -70,6 +74,8 @@ extern const struct xlat v4l2_buf_flags[];
 extern const struct xlat v4l2_streaming_capabilities[];
 extern const struct xlat v4l2_capture_modes[];
 extern const struct xlat v4l2_input_types[];
+extern const struct xlat v4l2_control_classes[];
+extern const struct xlat v4l2_control_id_bases[];
 extern const struct xlat v4l2_control_ids[];
 extern const struct xlat v4l2_tuner_types[];
 extern const struct xlat v4l2_tuner_capabilities[];
@@ -77,6 +83,5 @@ extern const struct xlat v4l2_tuner_rxsubchanses[];
 extern const struct xlat v4l2_tuner_audmodes[];
 extern const struct xlat v4l2_control_types[];
 extern const struct xlat v4l2_control_flags[];
-extern const struct xlat v4l2_control_classes[];
 extern const struct xlat v4l2_framesize_types[];
 extern const struct xlat v4l2_frameinterval_types[];
