@@ -278,7 +278,8 @@ Statistics:\n\
 \n\
 Filtering:\n\
   -e expr        a qualifying expression: option=[!]all or option=[!]val1[,val2]...\n\
-     options:    trace, abbrev, verbose, raw, signal, read, write, fault\n\
+     options:    trace, abbrev, verbose, raw, signal, read, write, fault, inject, kvm"
+"\n\
   -P path        trace accesses to path\n\
 \n\
 Tracing:\n\
@@ -819,6 +820,10 @@ droptcb(struct tcb *tcp)
 		unwind_tcb_fin(tcp);
 #endif
 
+#ifdef HAVE_LINUX_KVM_H
+	kvm_vcpu_info_free(tcp);
+#endif
+
 	if (tcp->mmap_cache)
 		tcp->mmap_cache->free_fn(tcp, __func__);
 
