@@ -33,7 +33,7 @@
  */
 
 #include "defs.h"
-#include "mmap_cache.h"
+#include "mmap_notify.h"
 #include "native_defs.h"
 #include "ptrace.h"
 #include "nsig.h"
@@ -57,10 +57,6 @@
 # define PTRACE_SETREGS PTRACE_SETREGS64
 #endif
 
-#ifndef NT_PRSTATUS
-# define NT_PRSTATUS 1
-#endif
-
 #include "syscall.h"
 #include "xstring.h"
 
@@ -90,6 +86,13 @@ static const struct_sysent sysent2[] = {
 /* Now undef them since short defines cause wicked namespace pollution. */
 #include "sysent_shorthand_undefs.h"
 
+const char *const errnoent[] = {
+#include "errnoent.h"
+};
+const char *const signalent[] = {
+#include "signalent.h"
+
+};
 /*
  * `ioctlent[012].h' files are automatically generated by the auxiliary
  * program `ioctlsort', such that the list is sorted by the `code' field.
@@ -98,23 +101,11 @@ static const struct_sysent sysent2[] = {
  * in "/usr/include".
  */
 
-const char *const errnoent0[] = {
-#include "errnoent.h"
-};
-const char *const signalent0[] = {
-#include "signalent.h"
-};
 const struct_ioctlent ioctlent0[] = {
 #include "ioctlent0.h"
 };
 
 #if SUPPORTED_PERSONALITIES > 1
-static const char *const errnoent1[] = {
-# include "errnoent1.h"
-};
-static const char *const signalent1[] = {
-# include "signalent1.h"
-};
 static const struct_ioctlent ioctlent1[] = {
 # include "ioctlent1.h"
 };
@@ -129,12 +120,6 @@ static const struct_printers printers1 = {
 #endif
 
 #if SUPPORTED_PERSONALITIES > 2
-static const char *const errnoent2[] = {
-# include "errnoent2.h"
-};
-static const char *const signalent2[] = {
-# include "signalent2.h"
-};
 static const struct_ioctlent ioctlent2[] = {
 # include "ioctlent2.h"
 };
@@ -154,26 +139,6 @@ enum {
 #endif
 };
 
-enum {
-	nerrnos0 = ARRAY_SIZE(errnoent0)
-#if SUPPORTED_PERSONALITIES > 1
-	, nerrnos1 = ARRAY_SIZE(errnoent1)
-# if SUPPORTED_PERSONALITIES > 2
-	, nerrnos2 = ARRAY_SIZE(errnoent2)
-# endif
-#endif
-};
-
-enum {
-	nsignals0 = ARRAY_SIZE(signalent0)
-#if SUPPORTED_PERSONALITIES > 1
-	, nsignals1 = ARRAY_SIZE(signalent1)
-# if SUPPORTED_PERSONALITIES > 2
-	, nsignals2 = ARRAY_SIZE(signalent2)
-# endif
-#endif
-};
-
 enum {
 	nioctlents0 = ARRAY_SIZE(ioctlent0)
 #if SUPPORTED_PERSONALITIES > 1
@@ -186,15 +151,13 @@ enum {
 
 #if SUPPORTED_PERSONALITIES > 1
 const struct_sysent *sysent = sysent0;
-const char *const *errnoent = errnoent0;
-const char *const *signalent = signalent0;
 const struct_ioctlent *ioctlent = ioctlent0;
 const struct_printers *printers = &printers0;
 #endif
 
+const unsigned int nerrnos = ARRAY_SIZE(errnoent);
+const unsigned int nsignals = ARRAY_SIZE(signalent);
 unsigned nsyscalls = nsyscalls0;
-unsigned nerrnos = nerrnos0;
-unsigned nsignals = nsignals0;
 unsigned nioctlents = nioctlents0;
 
 const unsigned int nsyscall_vec[SUPPORTED_PERSONALITIES] = {
@@ -228,6 +191,18 @@ const char *const personality_names[] =
 # endif
 	;
 
+const char *const personality_designators[] =
+# if defined X86_64
+	{ "64", "32", "x32" }
+# elif defined X32
+	{ "x32", "32" }
+# elif SUPPORTED_PERSONALITIES == 2
+	{ "64", "32" }
+# else
+	{ STRINGIFY_VAL(__WORDSIZE) }
+# endif
+	;
+
 #if SUPPORTED_PERSONALITIES > 1
 
 unsigned current_personality;
@@ -269,33 +244,21 @@ set_personality(unsigned int personality)
 
 	switch (personality) {
 	case 0:
-		errnoent = errnoent0;
-		nerrnos = nerrnos0;
 		ioctlent = ioctlent0;
 		nioctlents = nioctlents0;
-		signalent = signalent0;
-		nsignals = nsignals0;
 		printers = &printers0;
 		break;
 
 	case 1:
-		errnoent = errnoent1;
-		nerrnos = nerrnos1;
 		ioctlent = ioctlent1;
 		nioctlents = nioctlents1;
-		signalent = signalent1;
-		nsignals = nsignals1;
 		printers = &printers1;
 		break;
 
 # if SUPPORTED_PERSONALITIES > 2
 	case 2:
-		errnoent = errnoent2;
-		nerrnos = nerrnos2;
 		ioctlent = ioctlent2;
 		nioctlents = nioctlents2;
-		signalent = signalent2;
-		nsignals = nsignals2;
 		printers = &printers2;
 		break;
 # endif
@@ -499,6 +462,18 @@ err_name(unsigned long err)
 	return NULL;
 }
 
+static void
+print_err_ret(kernel_ulong_t ret, unsigned long u_error)
+{
+	const char *u_error_str = err_name(u_error);
+
+	if (u_error_str)
+		tprintf("= %" PRI_kld " %s (%s)",
+			ret, u_error_str, strerror(u_error));
+	else
+		tprintf("= %" PRI_kld " (errno %lu)", ret, u_error);
+}
+
 static long get_regs(struct tcb *);
 static int get_syscall_args(struct tcb *);
 static int get_syscall_result(struct tcb *);
@@ -544,9 +519,18 @@ tamper_with_syscall_entering(struct tcb *tcp, unsigned int *signo)
 	if (!recovering(tcp)) {
 		if (opts->data.flags & INJECT_F_SIGNAL)
 			*signo = opts->data.signo;
-		if (opts->data.flags & (INJECT_F_ERROR | INJECT_F_RETVAL) &&
-		    !arch_set_scno(tcp, -1))
-			tcp->flags |= TCB_TAMPERED;
+		if (opts->data.flags & (INJECT_F_ERROR | INJECT_F_RETVAL)) {
+			kernel_long_t scno =
+				(opts->data.flags & INJECT_F_SYSCALL)
+				? (kernel_long_t) shuffle_scno(opts->data.scno)
+				: -1;
+
+			if (!arch_set_scno(tcp, scno)) {
+				tcp->flags |= TCB_TAMPERED;
+				if (scno != -1)
+					tcp->flags |= TCB_TAMPERED_NO_FAIL;
+			}
+		}
 		if (opts->data.flags & INJECT_F_DELAY_ENTER)
 			delay_tcb(tcp, opts->data.delay_idx, true);
 		if (opts->data.flags & INJECT_F_DELAY_EXIT)
@@ -569,10 +553,11 @@ tamper_with_syscall_exiting(struct tcb *tcp)
 	if (!syscall_tampered(tcp))
 		return 0;
 
-	if (!syserror(tcp)) {
-		error_msg("Failed to tamper with process %d: got no error "
-			  "(return value %#" PRI_klx ")",
-			  tcp->pid, tcp->u_rval);
+	if (!syserror(tcp) ^ !!syscall_tampered_nofail(tcp)) {
+		error_msg("Failed to tamper with process %d: unexpectedly got"
+			  " %serror (return value %#" PRI_klx ", error %lu)",
+			  tcp->pid, syscall_tampered_nofail(tcp) ? "" : "no ",
+			  tcp->u_rval, tcp->u_error);
 
 		return 1;
 	}
@@ -705,7 +690,7 @@ syscall_entering_trace(struct tcb *tcp, unsigned int *sig)
 		return 0;
 	}
 
-#ifdef USE_LIBUNWIND
+#ifdef ENABLE_STACKTRACE
 	if (stack_trace_enabled) {
 		if (tcp->s_ent->sys_flags & STACKTRACE_CAPTURE_ON_ENTER)
 			unwind_tcb_capture(tcp);
@@ -744,10 +729,8 @@ syscall_exiting_decode(struct tcb *tcp, struct timespec *pts)
 	if ((Tflag || cflag) && !(filtered(tcp) || hide_log(tcp)))
 		clock_gettime(CLOCK_MONOTONIC, pts);
 
-	if (mmap_cache_is_enabled()) {
-		if (tcp->s_ent->sys_flags & STACKTRACE_INVALIDATE_CACHE)
-			mmap_cache_invalidate(tcp);
-	}
+	if (tcp->s_ent->sys_flags & MEMORY_MAPPING_CHANGE)
+		mmap_notify_report(tcp);
 
 	if (filtered(tcp) || hide_log(tcp))
 		return 0;
@@ -821,20 +804,17 @@ syscall_exiting_trace(struct tcb *tcp, struct timespec *ts, int res)
 
 	tprints(") ");
 	tabto();
-	unsigned long u_error = tcp->u_error;
 
 	if (raw(tcp)) {
-		if (u_error) {
-			tprintf("= -1 (errno %lu)", u_error);
-		} else {
+		if (tcp->u_error)
+			print_err_ret(tcp->u_rval, tcp->u_error);
+		else
 			tprintf("= %#" PRI_klx, tcp->u_rval);
-		}
+
 		if (syscall_tampered(tcp))
 			tprints(" (INJECTED)");
-	} else if (!(sys_res & RVAL_NONE) && u_error) {
-		const char *u_error_str;
-
-		switch (u_error) {
+	} else if (!(sys_res & RVAL_NONE) && tcp->u_error) {
+		switch (tcp->u_error) {
 		/* Blocked signals do not interrupt any syscalls.
 		 * In this case syscalls don't return ERESTARTfoo codes.
 		 *
@@ -889,13 +869,7 @@ syscall_exiting_trace(struct tcb *tcp, struct timespec *ts, int res)
 			tprints("= ? ERESTART_RESTARTBLOCK (Interrupted by signal)");
 			break;
 		default:
-			u_error_str = err_name(u_error);
-			if (u_error_str)
-				tprintf("= %" PRI_kld " %s (%s)", tcp->u_rval,
-					u_error_str, strerror(u_error));
-			else
-				tprintf("= %" PRI_kld " %lu (%s)", tcp->u_rval,
-					u_error, strerror(u_error));
+			print_err_ret(tcp->u_rval, tcp->u_error);
 			break;
 		}
 		if (syscall_tampered(tcp))
@@ -959,7 +933,7 @@ syscall_exiting_trace(struct tcb *tcp, struct timespec *ts, int res)
 	dumpio(tcp);
 	line_ended();
 
-#ifdef USE_LIBUNWIND
+#ifdef ENABLE_STACKTRACE
 	if (stack_trace_enabled)
 		unwind_tcb_print(tcp);
 #endif
@@ -1003,6 +977,10 @@ restore_cleared_syserror(struct tcb *tcp)
 	tcp->u_error = saved_u_error;
 }
 
+#define XLAT_MACROS_ONLY
+# include "xlat/nt_descriptor_types.h"
+#undef XLAT_MACROS_ONLY
+
 #include "arch_regs.c"
 
 #if HAVE_ARCH_GETRVAL2
@@ -1281,8 +1259,9 @@ get_syscall_result(struct tcb *tcp)
 		return -1;
 	tcp->u_error = 0;
 	get_error(tcp,
-		  !(tcp->s_ent->sys_flags & SYSCALL_NEVER_FAILS)
-			|| syscall_tampered(tcp));
+		  (!(tcp->s_ent->sys_flags & SYSCALL_NEVER_FAILS)
+			|| syscall_tampered(tcp))
+                  && !syscall_tampered_nofail(tcp));
 
 	return 1;
 }
