@@ -41,7 +41,7 @@
 # define PU	TRACE_PURE
 # define NF	SYSCALL_NEVER_FAILS
 # define MA	MAX_ARGS
-# define SI	STACKTRACE_INVALIDATE_CACHE
+# define SI	MEMORY_MAPPING_CHANGE
 # define SE	STACKTRACE_CAPTURE_ON_ENTER
 # define CST	COMPAT_SYSCALL_TYPES
 /* SEN(a) is defined elsewhere */
