@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2016 Mikulas Patocka <mpatocka@redhat.com>
  * Copyright (c) 2016 Eugene Syromyatnikov <evgsyr@gmail.com>
- * Copyright (c) 2016-2017 The strace developers.
+ * Copyright (c) 2016-2018 The strace developers.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +35,6 @@
 
 # include <errno.h>
 # include <inttypes.h>
-# include <stdbool.h>
 # include <stdio.h>
 # include <stddef.h>
 # include <string.h>
@@ -54,6 +53,10 @@
 # define ALIGNED_OFFSET(t_, m_) \
 	ALIGNED_SIZE(offsetof(t_, m_), t_)
 
+#  ifndef DM_DEV_ARM_POLL
+#   define DM_DEV_ARM_POLL     _IOWR(DM_IOCTL, 0x10, struct dm_ioctl)
+#  endif
+
 static const char str129[] = STR32 STR32 STR32 STR32 "6";
 
 static const __u64 dts_sector_base = (__u64) 0xdeadca75facef157ULL;
@@ -139,7 +142,7 @@ init_dm_target_spec(struct dm_target_spec *ptr, uint32_t id)
 	ptr->length       = dts_length_base + dts_length_step * id;
 	ptr->status       = dts_status_base + dts_status_step * id;
 
-	strncpy(ptr->target_type, str129 +
+	memcpy(ptr->target_type, str129 +
 		id % (sizeof(str129) - sizeof(ptr->target_type)),
 		id % (sizeof(ptr->target_type) + 1));
 	if (id % (sizeof(ptr->target_type) + 1) < sizeof(ptr->target_type))
@@ -182,6 +185,7 @@ main(void)
 		{ ARG_STR(DM_TABLE_CLEAR),   false },
 		{ ARG_STR(DM_TABLE_DEPS),    true  },
 		{ ARG_STR(DM_TABLE_STATUS),  true  },
+		{ ARG_STR(DM_DEV_ARM_POLL),  false },
 	};
 
 	struct dm_ioctl *unaligned_dm_arg =
@@ -247,11 +251,11 @@ main(void)
 
 	/* Unterminated name/uuid */
 	init_s(dm_arg, min_sizeof_dm_ioctl, 0);
-	strncpy(dm_arg->name, str129, sizeof(dm_arg->name));
-	strncpy(dm_arg->uuid, str129, sizeof(dm_arg->uuid));
+	memcpy(dm_arg->name, str129, sizeof(dm_arg->name));
+	memcpy(dm_arg->uuid, str129, sizeof(dm_arg->uuid));
 	ioctl(-1, DM_VERSION, dm_arg);
 	printf("ioctl(-1, DM_VERSION, {version=4.1.2, data_size=%zu, "
-	       "dev=makedev(18, 52), name=\"%.127s\", uuid=\"%.128s\", "
+	       "dev=makedev(18, 52), name=\"%.127s\"..., uuid=\"%.128s\"..., "
 	       "flags=0}) = -1 EBADF (%m)\n",
 	       min_sizeof_dm_ioctl, str129, str129);
 
@@ -506,7 +510,7 @@ main(void)
 					 target##id_next) - \
 				offsetof(struct dm_table_open_test, \
 					 target##id); \
-			strncpy(dm_arg_open3->param##id, str129 + id * 2, id); \
+			memcpy(dm_arg_open3->param##id, str129 + id * 2, id); \
 			dm_arg_open3->param##id[id] = '\0'; \
 		} while (0)
 	#define PRINT_DM_TARGET(id) \
