@@ -2,7 +2,7 @@
  * Check decoding of struct msghdr ancillary data.
  *
  * Copyright (c) 2016 Dmitry V. Levin <ldv@altlinux.org>
- * Copyright (c) 2016-2017 The strace developers.
+ * Copyright (c) 2016-2018 The strace developers.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -132,7 +132,7 @@ test_scm_rights1(struct msghdr *const mh,
 		print_fds(cmsg, src_len);
 		printf("}");
 		if (aligned_cms_len < msg_controllen)
-			printf(", %p", (void *) cmsg + aligned_cms_len);
+			printf(", ... /* %p */", (void *) cmsg + aligned_cms_len);
 		printf("]");
 	}
 
@@ -195,7 +195,7 @@ test_scm_rights2(struct msghdr *const mh,
 	print_fds(cmsg[1], src1_len);
 	printf("}");
 	if (aligned_cms_len[1] < msg_controllen1)
-		printf(", %p", (void *) cmsg[1] + aligned_cms_len[1]);
+		printf(", ... /* %p */", (void *) cmsg[1] + aligned_cms_len[1]);
 	printf("]");
 
 	errno = saved_errno;
@@ -425,7 +425,7 @@ test_scm_security(struct msghdr *const mh,
 	print_security(cmsg, src_len);
 	printf("}");
 	if (aligned_cms_len < msg_controllen)
-		printf(", %p", (void *) cmsg + aligned_cms_len);
+		printf(", ... /* %p */", (void *) cmsg + aligned_cms_len);
 	printf("]");
 
 	errno = saved_errno;
@@ -544,11 +544,7 @@ test_ip_pktinfo(struct msghdr *const mh, void *const page,
 	cmsg->cmsg_type = cmsg_type;
 
 	struct in_pktinfo *const info = (struct in_pktinfo *) CMSG_DATA(cmsg);
-#ifdef HAVE_IF_INDEXTONAME
-	info->ipi_ifindex = if_nametoindex("lo");
-#else
-	info->ipi_ifindex = 1;
-#endif
+	info->ipi_ifindex = ifindex_lo();
 	info->ipi_spec_dst.s_addr = inet_addr("1.2.3.4");
 	info->ipi_addr.s_addr = inet_addr("5.6.7.8");
 
@@ -563,12 +559,7 @@ test_ip_pktinfo(struct msghdr *const mh, void *const page,
 	       ", ipi_addr=inet_addr(\"%s\")}}]"
 	       ", msg_controllen=%u, msg_flags=0}, 0) = %d %s (%m)\n",
 	       (unsigned) cmsg->cmsg_len, cmsg_type_str,
-#ifdef HAVE_IF_INDEXTONAME
-	       "if_nametoindex(\"lo\")",
-#else
-	       "1",
-#endif
-	       "1.2.3.4", "5.6.7.8", len, rc, errno2name());
+	       IFINDEX_LO_STR, "1.2.3.4", "5.6.7.8", len, rc, errno2name());
 }
 
 static void
