@@ -2,7 +2,7 @@
  * Check decoding of sockaddr structures
  *
  * Copyright (c) 2016 Dmitry V. Levin <ldv@altlinux.org>
- * Copyright (c) 2016-2017 The strace developers.
+ * Copyright (c) 2016-2018 The strace developers.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,11 +50,6 @@
 # include <bluetooth/sco.h>
 #endif
 
-#ifdef HAVE_IF_INDEXTONAME
-/* <linux/if.h> used to conflict with <net/if.h> */
-extern unsigned int if_nametoindex(const char *);
-#endif
-
 static void
 check_un(void)
 {
@@ -186,19 +181,17 @@ check_in6_linklocal(struct sockaddr_in6 *const in6, const char *const h_addr)
 	       ntohs(in6->sin6_port), h_addr,
 	       ntohl(in6->sin6_flowinfo), in6->sin6_scope_id, len, ret);
 
-#ifdef HAVE_IF_INDEXTONAME
-	in6->sin6_scope_id = if_nametoindex("lo");
+	in6->sin6_scope_id = ifindex_lo();
 	if (in6->sin6_scope_id) {
 		ret = connect(-1, (void *) in6, len);
 		printf("connect(-1, {sa_family=AF_INET6, sin6_port=htons(%hu)"
 		       ", inet_pton(AF_INET6, \"%s\", &sin6_addr)"
 		       ", sin6_flowinfo=htonl(%u)"
-		       ", sin6_scope_id=if_nametoindex(\"lo\")}, %u)"
+		       ", sin6_scope_id=%s}, %u)"
 		       " = %d EBADF (%m)\n",
-		       ntohs(in6->sin6_port), h_addr,
-		       ntohl(in6->sin6_flowinfo), len, ret);
+		       ntohs(in6->sin6_port), h_addr, ntohl(in6->sin6_flowinfo),
+		       IFINDEX_LO_STR, len, ret);
 	}
-#endif
 }
 
 static void
@@ -371,19 +364,16 @@ check_ll(void)
 	       ", sll_pkttype=PACKET_HOST, sll_halen=0}, %u)"
 	       " = %d EBADF (%m)\n", c_ll.sll_ifindex, len, ret);
 
-#ifdef HAVE_IF_INDEXTONAME
-	const int id = if_nametoindex("lo");
-	if (id) {
-		((struct sockaddr_ll *) ll)->sll_ifindex = id;
+	((struct sockaddr_ll *) ll)->sll_ifindex = ifindex_lo();
+	if (((struct sockaddr_ll *) ll)->sll_ifindex) {
 		ret = connect(-1, ll, len);
 		printf("connect(-1, {sa_family=AF_PACKET"
 		       ", sll_protocol=htons(ETH_P_ALL)"
-		       ", sll_ifindex=if_nametoindex(\"lo\")"
+		       ", sll_ifindex=%s"
 		       ", sll_hatype=ARPHRD_ETHER"
 		       ", sll_pkttype=PACKET_HOST, sll_halen=0}, %u)"
-		       " = %d EBADF (%m)\n", len, ret);
+		       " = %d EBADF (%m)\n", IFINDEX_LO_STR, len, ret);
 	}
-#endif
 }
 
 #ifdef HAVE_BLUETOOTH_BLUETOOTH_H
@@ -446,25 +436,86 @@ check_l2(void)
 {
 	const unsigned short h_psm = 12345;
 	const unsigned short h_cid = 13579;
-	const struct sockaddr_l2 c_l2 = {
+	struct sockaddr_l2 c_l2 = {
 		.l2_family = AF_BLUETOOTH,
 		.l2_psm = htobs(h_psm),
 		.l2_bdaddr.b = "abcdef",
 		.l2_cid = htobs(h_cid),
-		.l2_bdaddr_type = 42
+#ifdef HAVE_STRUCT_SOCKADDR_L2_L2_BDADDR_TYPE
+		.l2_bdaddr_type = 0xce,
+#endif
 	};
 	void *l2 = tail_memdup(&c_l2, sizeof(c_l2));
 	unsigned int len = sizeof(c_l2);
+
 	int ret = connect(-1, l2, len);
 	printf("connect(-1, {sa_family=AF_BLUETOOTH"
-	       ", l2_psm=htobs(%hu)"
+	       ", l2_psm=htobs(L2CAP_PSM_DYN_START + %hu)"
 	       ", l2_bdaddr=%02x:%02x:%02x:%02x:%02x:%02x"
-	       ", l2_cid=htobs(%hu), l2_bdaddr_type=%u}"
-	       ", %u) = %d EBADF (%m)\n", h_psm,
+	       ", l2_cid=htobs(L2CAP_CID_DYN_START + %hu)"
+#ifdef HAVE_STRUCT_SOCKADDR_L2_L2_BDADDR_TYPE
+	       ", l2_bdaddr_type=0xce /* BDADDR_??? */"
+#endif
+	       "}, %u) = %d EBADF (%m)\n",
+	       (short) (h_psm - 0x1001),
 	       c_l2.l2_bdaddr.b[0], c_l2.l2_bdaddr.b[1],
 	       c_l2.l2_bdaddr.b[2], c_l2.l2_bdaddr.b[3],
 	       c_l2.l2_bdaddr.b[4], c_l2.l2_bdaddr.b[5],
-	       h_cid, c_l2.l2_bdaddr_type, len, ret);
+	       (short) (h_cid - 0x40), len, ret);
+
+	c_l2.l2_psm = htobs(1);
+	c_l2.l2_cid = htobs(1);
+#ifdef HAVE_STRUCT_SOCKADDR_L2_L2_BDADDR_TYPE
+	c_l2.l2_bdaddr_type = BDADDR_LE_RANDOM;
+#endif
+	memcpy(l2, &c_l2, sizeof(c_l2));
+	ret = connect(-1, l2, len);
+	printf("connect(-1, {sa_family=AF_BLUETOOTH"
+	       ", l2_psm=htobs(L2CAP_PSM_SDP)"
+	       ", l2_bdaddr=%02x:%02x:%02x:%02x:%02x:%02x"
+	       ", l2_cid=htobs(L2CAP_CID_SIGNALING)"
+#ifdef HAVE_STRUCT_SOCKADDR_L2_L2_BDADDR_TYPE
+	       ", l2_bdaddr_type=BDADDR_LE_RANDOM"
+#endif
+	       "}, %u) = %d EBADF (%m)\n",
+	       c_l2.l2_bdaddr.b[0], c_l2.l2_bdaddr.b[1],
+	       c_l2.l2_bdaddr.b[2], c_l2.l2_bdaddr.b[3],
+	       c_l2.l2_bdaddr.b[4], c_l2.l2_bdaddr.b[5],
+	       len, ret);
+
+	c_l2.l2_psm = htobs(0xbad);
+	c_l2.l2_cid = htobs(8);
+#ifdef HAVE_STRUCT_SOCKADDR_L2_L2_BDADDR_TYPE
+	c_l2.l2_bdaddr_type = 3;
+#endif
+	memcpy(l2, &c_l2, sizeof(c_l2));
+	ret = connect(-1, l2, len);
+	printf("connect(-1, {sa_family=AF_BLUETOOTH"
+	       ", l2_psm=htobs(0xbad /* L2CAP_PSM_??? */)"
+	       ", l2_bdaddr=%02x:%02x:%02x:%02x:%02x:%02x"
+	       ", l2_cid=htobs(0x8 /* L2CAP_CID_??? */)"
+#ifdef HAVE_STRUCT_SOCKADDR_L2_L2_BDADDR_TYPE
+	       ", l2_bdaddr_type=0x3 /* BDADDR_??? */"
+#endif
+	       "}, %u) = %d EBADF (%m)\n",
+	       c_l2.l2_bdaddr.b[0], c_l2.l2_bdaddr.b[1],
+	       c_l2.l2_bdaddr.b[2], c_l2.l2_bdaddr.b[3],
+	       c_l2.l2_bdaddr.b[4], c_l2.l2_bdaddr.b[5],
+	       len, ret);
+
+	c_l2.l2_psm = htobs(0x10ff);
+	c_l2.l2_cid = htobs(0xffff);
+	memcpy(l2, &c_l2, 12);
+	ret = connect(-1, l2, 12);
+	printf("connect(-1, {sa_family=AF_BLUETOOTH"
+	       ", l2_psm=htobs(L2CAP_PSM_AUTO_END)"
+	       ", l2_bdaddr=%02x:%02x:%02x:%02x:%02x:%02x"
+	       ", l2_cid=htobs(L2CAP_CID_DYN_END)"
+	       "}, 12) = %d EBADF (%m)\n",
+	       c_l2.l2_bdaddr.b[0], c_l2.l2_bdaddr.b[1],
+	       c_l2.l2_bdaddr.b[2], c_l2.l2_bdaddr.b[3],
+	       c_l2.l2_bdaddr.b[4], c_l2.l2_bdaddr.b[5],
+	       ret);
 }
 #endif
 
