@@ -2,6 +2,7 @@
  * This file is part of ioctl_evdev strace test.
  *
  * Copyright (c) 2016 Dmitry V. Levin <ldv@altlinux.org>
+ * Copyright (c) 2016-2018 The strace developers.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,18 +39,15 @@
 # include <sys/ioctl.h>
 # include <linux/input.h>
 
+# ifndef EV_SW
+#  define EV_SW 5
+# endif
+# ifndef ABS_MT_TOOL_Y
+#  define ABS_MT_TOOL_Y 0x3d
+# endif
+
 static const unsigned int magic = 0xdeadbeef;
-static const unsigned long lmagic = (unsigned long) 0xdeadbeefbadc0ded;
-
-static void
-init_magic(void *addr, const unsigned int size)
-{
-	unsigned int *p = addr;
-	const unsigned int *end = addr + size - sizeof(int);
-
-	for (; p <= end; ++p)
-		*(unsigned int *) p = magic;
-}
+static const unsigned long lmagic = (unsigned long) 0xdeadbeefbadc0dedULL;
 
 # if VERBOSE
 static void
@@ -76,9 +74,13 @@ print_ffe_common(const struct ff_effect *const ffe, const char *const type_str)
 # endif /* VERBOSE */
 }
 
-# define TEST_NULL_ARG(cmd) \
-	ioctl(-1, cmd, 0); \
-	printf("ioctl(-1, %s, NULL) = -1 EBADF (%m)\n", #cmd)
+# define TEST_NULL_ARG_EX(cmd, str)						\
+	do {								\
+		ioctl(-1, cmd, 0);					\
+		printf("ioctl(-1, %s, NULL) = -1 EBADF (%m)\n", str);	\
+	} while (0)
+
+# define TEST_NULL_ARG(cmd) TEST_NULL_ARG_EX(cmd, #cmd)
 
 int
 main(void)
@@ -124,14 +126,24 @@ main(void)
 	TEST_NULL_ARG(EVIOCGABS(ABS_X));
 	TEST_NULL_ARG(EVIOCSABS(ABS_X));
 
+	TEST_NULL_ARG_EX(EVIOCGABS(0xe), "EVIOCGABS(0xe /* ABS_??? */)");
+	TEST_NULL_ARG_EX(EVIOCSABS(0xe), "EVIOCSABS(0xe /* ABS_??? */)");
+
+	TEST_NULL_ARG(EVIOCGABS(ABS_MT_TOOL_Y));
+	TEST_NULL_ARG(EVIOCSABS(ABS_MT_TOOL_Y));
+
+	TEST_NULL_ARG_EX(EVIOCGABS(0x3e), "EVIOCGABS(0x3e /* ABS_??? */)");
+	TEST_NULL_ARG_EX(EVIOCSABS(0x3e), "EVIOCSABS(0x3e /* ABS_??? */)");
+
+	TEST_NULL_ARG_EX(EVIOCGABS(0x3f), "EVIOCGABS(0x3f /* ABS_??? */)");
+	TEST_NULL_ARG_EX(EVIOCSABS(0x3f), "EVIOCSABS(0x3f /* ABS_??? */)");
+
 	TEST_NULL_ARG(EVIOCGBIT(EV_SYN, 0));
 	TEST_NULL_ARG(EVIOCGBIT(EV_KEY, 1));
 	TEST_NULL_ARG(EVIOCGBIT(EV_REL, 2));
 	TEST_NULL_ARG(EVIOCGBIT(EV_ABS, 3));
 	TEST_NULL_ARG(EVIOCGBIT(EV_MSC, 4));
-# ifdef EV_SW
 	TEST_NULL_ARG(EVIOCGBIT(EV_SW, 5));
-# endif
 	TEST_NULL_ARG(EVIOCGBIT(EV_LED, 6));
 	TEST_NULL_ARG(EVIOCGBIT(EV_SND, 7));
 	TEST_NULL_ARG(EVIOCGBIT(EV_REP, 8));
@@ -139,6 +151,12 @@ main(void)
 	TEST_NULL_ARG(EVIOCGBIT(EV_PWR, 10));
 	TEST_NULL_ARG(EVIOCGBIT(EV_FF_STATUS, 11));
 
+	TEST_NULL_ARG_EX(EVIOCGBIT(0x6, 12), "EVIOCGBIT(0x6 /* EV_??? */, 12)");
+	TEST_NULL_ARG_EX(EVIOCGBIT(0x18, 13),
+			 "EVIOCGBIT(0x18 /* EV_??? */, 13)");
+	TEST_NULL_ARG_EX(EVIOCGBIT(0x1f, 14),
+			 "EVIOCGBIT(0x1f /* EV_??? */, 14)");
+
 	ioctl(-1, EVIOCGBIT(EV_MAX, 42), 0);
 	printf("ioctl(-1, EVIOCGBIT(%#x /* EV_??? */, 42), NULL)"
 	       " = -1 EBADF (%m)\n", EV_MAX);
@@ -156,9 +174,9 @@ main(void)
 
 	const unsigned int size = get_page_size();
 	void *const page = tail_alloc(size);
-	init_magic(page, size);
+	fill_memory(page, size);
 
-	int *const val_int = tail_alloc(sizeof(*val_int));
+	TAIL_ALLOC_OBJECT_CONST_PTR(int, val_int);
 	*val_int = magic;
 
 # ifdef EVIOCSCLOCKID
@@ -182,8 +200,8 @@ main(void)
 	       pair_int[0], "KEY_ESC");
 
 # ifdef EVIOCSKEYCODE_V2
-	struct input_keymap_entry *const ike = tail_alloc(sizeof(*ike));
-	init_magic(ike, sizeof(*ike));
+	TAIL_ALLOC_OBJECT_CONST_PTR(struct input_keymap_entry, ike);
+	fill_memory(ike, sizeof(*ike));
 	ike->keycode = 2;
 
 	ioctl(-1, EVIOCSKEYCODE_V2, ike);
@@ -206,8 +224,8 @@ main(void)
 	printf("}) = -1 EBADF (%m)\n");
 # endif
 
-	struct ff_effect *const ffe = tail_alloc(sizeof(*ffe));
-	init_magic(ffe, sizeof(*ffe));
+	TAIL_ALLOC_OBJECT_CONST_PTR(struct ff_effect, ffe);
+	fill_memory(ffe, sizeof(*ffe));
 
 	ffe->type = FF_CONSTANT;
 	ioctl(-1, EVIOCSFF, ffe);
@@ -262,18 +280,22 @@ main(void)
 	printf("}) = -1 EBADF (%m)\n");
 #  endif
 
-	ioctl(-1, _IOC(_IOC_READ, 0x45, 0x01, 0xff), lmagic);
+	ioctl(-1, _IOC(_IOC_READ, 0x45, 0x1, 0xff), lmagic);
 	printf("ioctl(-1, %s, %#lx) = -1 EBADF (%m)\n",
-	       "_IOC(_IOC_READ, 0x45, 0x01, 0xff)", lmagic);
+	       "_IOC(_IOC_READ, 0x45, 0x1, 0xff)", lmagic);
 
-	ioctl(-1, _IOC(_IOC_WRITE, 0x45, 0x01, 0xff), lmagic);
+	ioctl(-1, _IOC(_IOC_WRITE, 0x45, 0x1, 0xff), lmagic);
 	printf("ioctl(-1, %s, %#lx) = -1 EBADF (%m)\n",
-	       "_IOC(_IOC_WRITE, 0x45, 0x01, 0xff)", lmagic);
+	       "_IOC(_IOC_WRITE, 0x45, 0x1, 0xff)", lmagic);
 
 	ioctl(-1, _IOC(_IOC_READ|_IOC_WRITE, 0x45, 0xfe, 0xff), lmagic);
 	printf("ioctl(-1, %s, %#lx) = -1 EBADF (%m)\n",
 	       "_IOC(_IOC_READ|_IOC_WRITE, 0x45, 0xfe, 0xff)", lmagic);
 
+	ioctl(-1, _IOC(_IOC_READ|_IOC_WRITE, 0x45, 0, 0), lmagic);
+	printf("ioctl(-1, %s, %#lx) = -1 EBADF (%m)\n",
+	       "_IOC(_IOC_READ|_IOC_WRITE, 0x45, 0, 0)", lmagic);
+
 	puts("+++ exited with 0 +++");
 	return 0;
 }
