@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2015 Eugene Syromyatnikov <evgsyr@gmail.com>
  * Copyright (c) 2015-2016 Dmitry V. Levin <ldv@altlinux.org>
+ * Copyright (c) 2015-2017 The strace developers.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,11 +50,11 @@ enum {
 };
 
 int
-main (void)
+main(void)
 {
 	struct timespec ts;
 	volatile int dummy = 0;
-	int i;
+	int i = 0;
 
 	pid_t pid = fork();
 	if (pid < 0)
@@ -67,6 +68,9 @@ main (void)
 		if (ts.tv_sec || ts.tv_nsec >= cputime_limit)
 			break;
 
+		if (i && !(ts.tv_sec || ts.tv_nsec))
+			error_msg_and_skip("clock_gettime(CLOCK_PROCESS_CPUTIME_ID, {0, 0})");
+
 		for (i = 0; i < NUM_USER_ITERS; ++i)
 			++dummy;
 	}
