@@ -2,7 +2,7 @@
  * This file is part of execve strace test.
  *
  * Copyright (c) 2015-2016 Dmitry V. Levin <ldv@altlinux.org>
- * Copyright (c) 2015-2017 The strace developers.
+ * Copyright (c) 2015-2018 The strace developers.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,17 +59,18 @@ main(void)
 
 	execve(FILENAME, tail_argv, tail_envp);
 	printf("execve(\"%s\""
-	       ", [\"%s\", \"%s\", \"%s\", %p, %p, %p, ???]"
+	       ", [\"%s\", \"%s\", \"%s\", %p, %p, %p, ... /* %p */]"
 #if VERBOSE
-	       ", [\"%s\", \"%s\", %p, %p, %p, ???]"
+	       ", [\"%s\", \"%s\", %p, %p, %p, ... /* %p */]"
 #else
 	       ", %p /* 5 vars, unterminated */"
 #endif
 	       ") = -1 ENOENT (%m)\n",
 	       Q_FILENAME, q_argv[0], q_argv[1], q_argv[2],
-	       argv[3], argv[4], argv[5]
+	       argv[3], argv[4], argv[5], (char *) tail_argv + sizeof(argv)
 #if VERBOSE
-	       , q_envp[0], q_envp[1], envp[2], envp[3], envp[4]
+	       , q_envp[0], q_envp[1], envp[2], envp[3], envp[4],
+	       (char *) tail_envp + sizeof(envp)
 #else
 	       , tail_envp
 #endif
