@@ -86,14 +86,14 @@ test_syscall(const unsigned long nr)
 	       a[0], a[1], a[2], a[3], a[4], a[5], rc);
 #else
 	printf("syscall_%#lx(%#llx, %#llx, %#llx, %#llx, %#llx, %#llx)"
-	       " = %ld (errno %d)\n", nr | SYSCALL_BIT,
+	       " = %ld ENOSYS (%m)\n", nr | SYSCALL_BIT,
 	       (unsigned long long) a[0],
 	       (unsigned long long) a[1],
 	       (unsigned long long) a[2],
 	       (unsigned long long) a[3],
 	       (unsigned long long) a[4],
 	       (unsigned long long) a[5],
-	       rc, errno);
+	       rc);
 #endif
 }
 
