@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/sh -eu
 #
 # Copyright (c) 2014-2015 Mike Frysinger <vapier@gentoo.org>
 # Copyright (c) 2014-2015 Dmitry V. Levin <ldv@altlinux.org>
@@ -45,7 +45,7 @@ cond_def()
 
 	local val
 	val="$(printf %s "$line" |
-		sed -r -n 's/^([^[:space:]]+).*$/\1/p')"
+		LC_ALL=C sed -r -n 's/^([[:alpha:]_][[:alnum:]_]*).*$/\1/p')"
 
 	local def
 	def="$(printf %s "${line}" |
@@ -53,7 +53,11 @@ cond_def()
 
 	if [ -n "$def" ]; then
 		cat <<-EOF
-		#if !(defined($val) || (defined(HAVE_DECL_$val) && HAVE_DECL_$val))
+		#if defined($val) || (defined(HAVE_DECL_$val) && HAVE_DECL_$val)
+		DIAG_PUSH_IGNORE_TAUTOLOGICAL_COMPARE
+		static_assert(($val) == ($def), "$val != $def");
+		DIAG_POP_IGNORE_TAUTOLOGICAL_COMPARE
+		#else
 		# define $val $def
 		#endif
 		EOF
@@ -65,6 +69,7 @@ print_xlat()
 	local val
 	val="$1"; shift
 
+	[ 1 = "$value_indexed" ] && printf " [%s] =" "${val}"
 	if [ -z "${val_type-}" ]; then
 		echo " XLAT(${val}),"
 	else
@@ -78,6 +83,7 @@ print_xlat_pair()
 	val="$1"; shift
 	str="$1"; shift
 
+	[ 1 = "$value_indexed" ] && printf " [%s] =" "${val}"
 	if [ -z "${val_type-}" ]; then
 		echo " XLAT_PAIR(${val}, \"${str}\"),"
 	else
@@ -123,13 +129,21 @@ gen_header()
 	local decl="extern const struct xlat ${name}[];"
 	local in_defs= in_mpers=
 
+	value_indexed=0
+
 	if grep -F -x "$decl" "$defs" > /dev/null; then
 		in_defs=1
 	elif grep -F -x "$decl" "$mpers" > /dev/null; then
 		in_mpers=1
 	fi
 
-	echo "/* Generated by $0 from $1; do not edit. */"
+	cat <<-EOF
+	/* Generated by $0 from $1; do not edit. */
+
+	#include "gcc_compat.h"
+	#include "static_assert.h"
+
+	EOF
 
 	local unconditional= line
 	# 1st pass: output directives.
@@ -151,6 +165,9 @@ gen_header()
 		'#val_type '*)
 			# to be processed during 2nd pass
 			;;
+		'#value_indexed')
+			value_indexed=1
+			;;
 		'#'*)
 			echo "${line}"
 			;;
@@ -161,35 +178,41 @@ gen_header()
 		esac
 	done < "$input"
 
-	echo
+	cat <<-EOF
+
+		#ifndef XLAT_MACROS_ONLY
+
+	EOF
+
 	if [ -n "$in_defs" ]; then
 		cat <<-EOF
-			#ifndef IN_MPERS
+			# ifndef IN_MPERS
 
 		EOF
 	elif [ -n "$in_mpers" ]; then
 		cat <<-EOF
-			#ifdef IN_MPERS
+			# ifdef IN_MPERS
 
 			${decl}
 
-			#else
+			# else
 
-			# if !(defined HAVE_M32_MPERS || defined HAVE_MX32_MPERS)
+			#  if !(defined HAVE_M32_MPERS || defined HAVE_MX32_MPERS)
 			static
-			# endif
+			#  endif
 		EOF
 	else
 		cat <<-EOF
-			#ifdef IN_MPERS
+			# ifdef IN_MPERS
 
-			# error static const struct xlat ${name} in mpers mode
+			#  error static const struct xlat ${name} in mpers mode
 
-			#else
+			# else
 
 			static
 		EOF
 	fi
+
 	echo "const struct xlat ${name}[] = {"
 
 	unconditional= val_type=
@@ -206,6 +229,8 @@ gen_header()
 		'#unconditional')
 			unconditional=1
 			;;
+		'#value_indexed')
+			;;
 		'#val_type '*)
 			val_type="${line#\#val_type }"
 			;;
@@ -236,7 +261,9 @@ gen_header()
 	cat <<-EOF
 		};
 
-		#endif /* !IN_MPERS */
+		# endif /* !IN_MPERS */
+
+		#endif /* !XLAT_MACROS_ONLY */
 	EOF
 	) >"${output}"
 }
